from enum import Enum

class Verbosetype(Enum):
    TYPE_DEFAULT="default"
    TYPE_RICH="rich"
    
class RichColors(Enum):
    RED = "red"
    GREEN = "green"
    YELLOW = "yellow"
    BLUE = "blue"
    MAGENTA = "magenta"
    CYAN = "cyan"
    WHITE = "white"
    BLACK = "black"